@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -34,39 +34,27 @@ class CControllerWidgetGraphView extends CControllerWidget {
 			'fields' => 'json',
 			'dynamic_hostid' => 'db hosts.hostid',
 			'content_width' => 'int32',
-			'content_height' => 'int32',
-			'only_footer' => 'in 1'
+			'content_height' => 'int32'
 		]);
 	}
 
 	protected function doAction() {
-		if ($this->getInput('only_footer', 0)) {
-			$this->setResponse(new CControllerResponseData([
-				'only_footer' => true,
-				'user' => [
-					'debug_mode' => $this->getDebugMode()
-				]
-			]));
-
-			return;
-		}
-
 		$fields = $this->getForm()->getFieldsData();
 
 		$uniqueid = $this->getInput('uniqueid');
 		$edit_mode = (int) $this->getInput('edit_mode', 0);
 
-		$width = $this->getInput('content_width', '100');
-		$height = $this->getInput('content_height', '100');
+		$width = (int) $this->getInput('content_width', 100);
+		$height = (int) $this->getInput('content_height', 100);
 
 		$dataid = 'graph_'.$uniqueid;
 		$containerid = 'graph_container_'.$uniqueid;
 		$dynamic_hostid = $this->getInput('dynamic_hostid', 0);
-		$dashboardid = $this->getInput('dashboardid', 0);
 		$resourceid = null;
-		$profileIdx = 'web.dashbrd';
-		$profileIdx2 = $dashboardid;
+		$profileIdx = 'web.dashbrd.filter';
+		$profileIdx2 = $this->getInput('dashboardid', 0);
 		$unavailable_object = false;
+		$header_label = '';
 
 		if ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_GRAPH && $fields['graphid']) {
 			$resource_type = SCREEN_RESOURCE_GRAPH;
@@ -77,7 +65,7 @@ class CControllerWidgetGraphView extends CControllerWidget {
 		}
 		elseif ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_SIMPLE_GRAPH && $fields['itemid']) {
 			$resource_type = SCREEN_RESOURCE_SIMPLE_GRAPH;
-			$resourceid = $fields['itemid'];
+			$resourceid = $fields['itemid'][0];
 			$graph_dims = getGraphDims();
 			$graph_dims['graphHeight'] = $height;
 			$graph_dims['width'] = $width;
@@ -86,15 +74,11 @@ class CControllerWidgetGraphView extends CControllerWidget {
 			$resource_type = null;
 			$graph_dims = getGraphDims();
 		}
+		$graph_dims['shiftYtop'] = CLineGraphDraw::DEFAULT_TOP_BOTTOM_PADDING;
 
-		// Prepare timeline details
-		$timeline = calculateTime([
+		$timeline = getTimeSelectorPeriod([
 			'profileIdx' => $profileIdx,
-			'profileIdx2' => $profileIdx2,
-			'updateProfile' => false,
-			'period' => null,
-			'stime' => null,
-			'isNow' => null
+			'profileIdx2' => $profileIdx2
 		]);
 
 		$time_control_data = [
@@ -103,8 +87,6 @@ class CControllerWidgetGraphView extends CControllerWidget {
 			'objDims' => $graph_dims,
 			'loadSBox' => 0,
 			'loadImage' => 1,
-			'periodFixed' => CProfile::get($profileIdx.'.timelinefixed', 1),
-			'sliderMaximumTimePeriod' => ZBX_MAX_PERIOD,
 			'reloadOnAdd' => 1,
 			'onDashboard' => 1
 		];
@@ -116,22 +98,35 @@ class CControllerWidgetGraphView extends CControllerWidget {
 			'timeline' => $timeline,
 			'resourcetype' => $resource_type,
 			'profileIdx' => $profileIdx,
-			'profileIdx2' => $profileIdx2,
-			'updateProfile' => false
+			'profileIdx2' => $profileIdx2
 		];
 
 		// Replace graph item by particular host item if dynamic items are used.
 		if ($fields['dynamic'] == WIDGET_DYNAMIC_ITEM && $dynamic_hostid && $resourceid) {
 			// Find same simple-graph item in selected $dynamic_hostid host.
 			if ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_SIMPLE_GRAPH) {
-				$items = get_same_item_for_host(['itemid' => $resourceid], [$dynamic_hostid]);
-				$item = reset($items);
-				$resourceid = ($item && array_key_exists('itemid', $item)) ? $item['itemid'] : null;
+				$src_items = API::Item()->get([
+					'output' => ['key_'],
+					'itemids' => $resourceid,
+					'webitems' => true
+				]);
 
-				if ($resourceid === null
-						|| !in_array($item['value_type'], [ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64])) {
+				$items = API::Item()->get([
+					'output' => ['itemid', 'hostid', 'name', 'key_'],
+					'selectHosts' => ['name'],
+					'hostids' => $dynamic_hostid,
+					'filter' => [
+						'key_' => $src_items[0]['key_'],
+						'value_type' => [ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64]
+					],
+					'webitems' => true
+				]);
+
+				$item = reset($items);
+				$resourceid = $items ? $item['itemid'] : null;
+
+				if ($resourceid === null) {
 					$unavailable_object = true;
-					$resourceid = null;
 				}
 			}
 			// Find requested host and change graph details.
@@ -185,6 +180,8 @@ class CControllerWidgetGraphView extends CControllerWidget {
 							$graph['ymin_type'] = GRAPH_YAXIS_TYPE_CALCULATED;
 						}
 					}
+
+					$graph['hosts'] = $hosts;
 				}
 
 				if ($graph) {
@@ -208,11 +205,26 @@ class CControllerWidgetGraphView extends CControllerWidget {
 			if (!$resourceid) {
 				$unavailable_object = true;
 			}
+			elseif ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_SIMPLE_GRAPH) {
+				$items = API::Item()->get([
+					'output' => ['name', 'key_', 'delay', 'hostid'],
+					'selectHosts' => ['name'],
+					'itemids' => $resourceid,
+					'filter' => ['value_type' => [ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64]],
+					'webitems' => true
+				]);
+				$item = reset($items);
+
+				if (!$item) {
+					$unavailable_object = true;
+				}
+			}
 			elseif ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_GRAPH) {
 				// get graph, used below
 				$graph = API::Graph()->get([
 					'graphids' => $resourceid,
-					'output' => API_OUTPUT_EXTEND
+					'output' => API_OUTPUT_EXTEND,
+					'selectHosts' => ['name']
 				]);
 				$graph = reset($graph);
 
@@ -222,23 +234,6 @@ class CControllerWidgetGraphView extends CControllerWidget {
 			}
 		}
 
-		if (!$unavailable_object && $fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_SIMPLE_GRAPH) {
-			$item = API::Item()->get([
-				'itemids' => $resourceid,
-				'output' => ['type', 'name', 'hostid', 'key_'],
-				'filter' => ['value_type' => [ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64]],
-				'selectHosts' => ['name'],
-				'webitems' => true
-			]);
-
-			if ($item) {
-				$item = reset($item);
-			}
-			else {
-				$unavailable_object = true;
-			}
-		}
-
 		if (!$unavailable_object) {
 			// Build graph action and data source links.
 			if ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_SIMPLE_GRAPH) {
@@ -251,17 +246,21 @@ class CControllerWidgetGraphView extends CControllerWidget {
 					$graph_src->setArgument('itemids[]', $resourceid);
 					$graph_src->setArgument('width', $width);
 					$graph_src->setArgument('height', $height);
+					$graph_src->setArgument('legend', $fields['show_legend']);
 				}
 				else {
 					$graph_src = new CUrl('chart3.php');
 				}
 
-				$graph_src->setArgument('period', $timeline['period']);
-				$graph_src->setArgument('stime', $timeline['stime']);
-				$graph_src->setArgument('isNow', $timeline['isNow']);
+				$graph_src->setArgument('from', $timeline['from']);
+				$graph_src->setArgument('to', $timeline['to']);
+
+				$item = CMacrosResolverHelper::resolveItemNames([$item])[0];
+				$header_label = $item['hosts'][0]['name'].NAME_DELIMITER.$item['name_expanded'];
 			}
 			elseif ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_GRAPH) {
 				$graph_src = '';
+				$header_label = $graph['hosts'][0]['name'].NAME_DELIMITER.$graph['name'];
 
 				if ($fields['dynamic'] == WIDGET_DYNAMIC_ITEM && $dynamic_hostid && $resourceid) {
 					$chart_file = ($graph['graphtype'] == GRAPH_TYPE_PIE || $graph['graphtype'] == GRAPH_TYPE_EXPLODED)
@@ -295,8 +294,6 @@ class CControllerWidgetGraphView extends CControllerWidget {
 						$graph_src = new CUrl('chart6.php');
 						$graph_src->setArgument('graphid', $resourceid);
 					}
-
-					$timeline['starttime'] = date(TIMESTAMP_FORMAT, get_min_itemclock_by_graphid($resourceid));
 				}
 				else {
 					if ($fields['dynamic'] == WIDGET_SIMPLE_ITEM || $graph_src === '') {
@@ -311,17 +308,15 @@ class CControllerWidgetGraphView extends CControllerWidget {
 
 				$graph_src->setArgument('width', $width);
 				$graph_src->setArgument('height', $height);
-				$graph_src->setArgument('legend', $graph['show_legend']);
-				$graph_src->setArgument('period', $timeline['period']);
-				$graph_src->setArgument('stime', $timeline['stime']);
-				$graph_src->setArgument('isNow', $timeline['isNow']);
+				$graph_src->setArgument('legend', ($fields['show_legend'] == 1 && $graph['show_legend'] == 1) ? 1 : 0);
+				$graph_src->setArgument('from', $timeline['from']);
+				$graph_src->setArgument('to', $timeline['to']);
 
 				if ($graph_dims['graphtype'] == GRAPH_TYPE_PIE || $graph_dims['graphtype'] == GRAPH_TYPE_EXPLODED) {
 					$graph_src->setArgument('graph3d', $graph['show_3d']);
 				}
 			}
 
-			$graph_src->setArgument('updateProfile', false);
 			$graph_src->setArgument('profileIdx', $profileIdx);
 			$graph_src->setArgument('profileIdx2', $profileIdx2);
 
@@ -329,17 +324,29 @@ class CControllerWidgetGraphView extends CControllerWidget {
 				$graph_src->setArgument('outer', '1');
 			}
 
+			$graph_src->setArgument('widget_view', '1');
 			$time_control_data['src'] = $graph_src->getUrl();
+
+			if ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_GRAPH) {
+				$item_graph_url = (new CUrl('charts.php'))->setArgument('graphid', $resourceid);
+			}
+			else {
+				$item_graph_url = (new CUrl('history.php'))->setArgument('itemids', [$resourceid]);
+			}
+			$item_graph_url
+				->setArgument('from', $timeline['from'])
+				->setArgument('to', $timeline['to']);
 		}
 
-		$this->setResponse(new CControllerResponseData([
-			'name' => $this->getInput('name', $this->getDefaultHeader()),
+		$response = [
+			'name' => $this->getInput('name', $header_label),
 			'graph' => [
 				'dataid' => $dataid,
 				'containerid' => $containerid,
 				'timestamp' => time(),
 				'unavailable_object' => $unavailable_object
 			],
+			'item_graph_url' => $unavailable_object ? '' : $item_graph_url,
 			'widget' => [
 				'uniqueid' => $uniqueid,
 				'initial_load' => (int) $this->getInput('initial_load', 0),
@@ -347,11 +354,11 @@ class CControllerWidgetGraphView extends CControllerWidget {
 			'time_control_data' => $time_control_data,
 			'timeline' => $timeline,
 			'fs_data' => $fs_data,
-			'dashboardid' => $dashboardid,
-			'only_footer' => false,
 			'user' => [
 				'debug_mode' => $this->getDebugMode()
 			]
-		]));
+		];
+
+		$this->setResponse(new CControllerResponseData($response));
 	}
 }
