@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -29,7 +29,6 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 		$this->setType(WIDGET_PROBLEMS);
 		$this->setValidationRules([
 			'name' => 'string',
-			'fullscreen' => 'in 0,1',
 			'fields' => 'json'
 		]);
 	}
@@ -44,30 +43,39 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 			'groupids' => getSubGroups($fields['groupids']),
 			'exclude_groupids' => getSubGroups($fields['exclude_groupids']),
 			'hostids' => $fields['hostids'],
-			'problem' => $fields['problem'],
+			'name' => $fields['problem'],
 			'severities' => $fields['severities'],
+			'evaltype' => $fields['evaltype'],
 			'tags' => $fields['tags'],
-			'maintenance' => $fields['maintenance'],
+			'show_suppressed' => $fields['show_suppressed'],
 			'unacknowledged' => $fields['unacknowledged']
-		], $config, true);
+		], $config);
 		list($sortfield, $sortorder) = self::getSorting($fields['sort_triggers']);
 		$data = CScreenProblem::sortData($data, $config, $sortfield, $sortorder);
 
-		$info = _n('%1$d of %3$d%2$s problem is shown', '%1$d of %3$d%2$s problems are shown',
-			min($fields['show_lines'], count($data['problems'])),
-			(count($data['problems']) > $config['search_limit']) ? '+' : '',
-			min($config['search_limit'], count($data['problems']))
-		);
+		if (count($data['problems']) > $fields['show_lines']) {
+			$info = _n('%1$d of %3$d%2$s problem is shown', '%1$d of %3$d%2$s problems are shown',
+				min($fields['show_lines'], count($data['problems'])),
+				(count($data['problems']) > $config['search_limit']) ? '+' : '',
+				min($config['search_limit'], count($data['problems']))
+			);
+		}
+		else {
+			$info = '';
+		}
 		$data['problems'] = array_slice($data['problems'], 0, $fields['show_lines'], true);
 
 		$data = CScreenProblem::makeData($data, [
 			'show' => $fields['show'],
 			'details' => 0
-		], $config, true);
+		], true);
 
 		if ($fields['show_tags']) {
-			$data['tags'] = makeEventsTags($data['problems']);
+			$data['tags'] = makeTags($data['problems'], true, 'eventid', $fields['show_tags'], $fields['tags'],
+				$fields['tag_name_format'], $fields['tag_priority']
+			);
 		}
+
 		if ($data['problems']) {
 			$data['triggers_hosts'] = getTriggersHostsList($data['triggers']);
 		}
@@ -76,17 +84,28 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 			'name' => $this->getInput('name', $this->getDefaultHeader()),
 			'fields' => [
 				'show' => $fields['show'],
-				'show_tags' => $fields['show_tags']
+				'show_lines' => $fields['show_lines'],
+				'show_tags' => $fields['show_tags'],
+				'show_timeline' => $fields['show_timeline'],
+				'tags' => $fields['tags'],
+				'tag_name_format' => $fields['tag_name_format'],
+				'tag_priority' => $fields['tag_priority']
 			],
 			'config' => [
-				'event_ack_enable' => $config['event_ack_enable'],
-				'blink_period' => timeUnitToSeconds($config['blink_period'])
+				'problem_unack_style' => $config['problem_unack_style'],
+				'problem_ack_style' => $config['problem_ack_style'],
+				'blink_period' => timeUnitToSeconds($config['blink_period']),
+				'severity_name_0' => $config['severity_name_0'],
+				'severity_name_1' => $config['severity_name_1'],
+				'severity_name_2' => $config['severity_name_2'],
+				'severity_name_3' => $config['severity_name_3'],
+				'severity_name_4' => $config['severity_name_4'],
+				'severity_name_5' => $config['severity_name_5']
 			],
 			'data' => $data,
 			'info' => $info,
 			'sortfield' => $sortfield,
 			'sortorder' => $sortorder,
-			'fullscreen' => $this->getInput('fullscreen', 0),
 			'user' => [
 				'debug_mode' => $this->getDebugMode()
 			]
@@ -113,10 +132,10 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 				return ['clock', ZBX_SORT_DOWN];
 
 			case SCREEN_SORT_TRIGGERS_SEVERITY_ASC:
-				return ['priority', ZBX_SORT_UP];
+				return ['severity', ZBX_SORT_UP];
 
 			case SCREEN_SORT_TRIGGERS_SEVERITY_DESC:
-				return ['priority', ZBX_SORT_DOWN];
+				return ['severity', ZBX_SORT_DOWN];
 
 			case SCREEN_SORT_TRIGGERS_HOST_NAME_ASC:
 				return ['host', ZBX_SORT_UP];
@@ -125,10 +144,10 @@ class CControllerWidgetProblemsView extends CControllerWidget {
 				return ['host', ZBX_SORT_DOWN];
 
 			case SCREEN_SORT_TRIGGERS_NAME_ASC:
-				return ['problem', ZBX_SORT_UP];
+				return ['name', ZBX_SORT_UP];
 
 			case SCREEN_SORT_TRIGGERS_NAME_DESC:
-				return ['problem', ZBX_SORT_DOWN];
+				return ['name', ZBX_SORT_DOWN];
 		}
 	}
 }
