@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -267,6 +267,20 @@ class CItemPrototype extends CItemGeneral {
 			$result = $this->unsetExtraFields($result, ['hostid'], $options['output']);
 		}
 
+		// Decode ITEM_TYPE_HTTPAGENT encoded fields.
+		$json = new CJson();
+
+		foreach ($result as &$item) {
+			if (array_key_exists('query_fields', $item)) {
+				$query_fields = ($item['query_fields'] !== '') ? $json->decode($item['query_fields'], true) : [];
+				$item['query_fields'] = $json->hasError() ? [] : $query_fields;
+			}
+
+			if (array_key_exists('headers', $item)) {
+				$item['headers'] = $this->headersStringToArray($item['headers']);
+			}
+		}
+
 		if (!$options['preservekeys']) {
 			$result = zbx_cleanHashes($result);
 		}
@@ -341,7 +355,32 @@ class CItemPrototype extends CItemGeneral {
 		foreach ($items as &$item) {
 			unset($item['itemid']);
 		}
-		$this->validateDependentItems($items, API::ItemPrototype());
+
+		$this->validateDependentItems($items, __METHOD__);
+
+		$json = new CJson();
+
+		foreach ($items as &$item) {
+			if ($item['type'] == ITEM_TYPE_HTTPAGENT) {
+				if (array_key_exists('query_fields', $item)) {
+					$item['query_fields'] = $item['query_fields'] ? $json->encode($item['query_fields']) : '';
+				}
+
+				if (array_key_exists('headers', $item)) {
+					$item['headers'] = $this->headersArrayToString($item['headers']);
+				}
+
+				if (array_key_exists('request_method', $item) && $item['request_method'] == HTTPCHECK_REQUEST_HEAD
+						&& !array_key_exists('retrieve_mode', $item)) {
+					$item['retrieve_mode'] = HTTPTEST_STEP_RETRIEVE_MODE_HEADERS;
+				}
+			}
+			else {
+				$item['query_fields'] = '';
+				$item['headers'] = '';
+			}
+		}
+		unset($item);
 
 		$this->createReal($items);
 		$this->inherit($items);
@@ -379,10 +418,10 @@ class CItemPrototype extends CItemGeneral {
 			}
 		}
 
-		DB::insert('item_discovery', $insertItemDiscovery);
+		DB::insertBatch('item_discovery', $insertItemDiscovery);
 
-		if (!empty($itemApplications)) {
-			DB::insert('items_applications', $itemApplications);
+		if ($itemApplications) {
+			DB::insertBatch('items_applications', $itemApplications);
 		}
 
 		$item_application_prototypes = [];
@@ -419,7 +458,7 @@ class CItemPrototype extends CItemGeneral {
 				$new_ids = [];
 
 				if ($application_prototypes_to_create) {
-					$new_ids = DB::insert('application_prototype', $application_prototypes_to_create);
+					$new_ids = DB::insertBatch('application_prototype', $application_prototypes_to_create);
 				}
 
 				$ids = array_merge($new_ids, zbx_objectValues($db_application_prototypes, 'application_prototypeid'));
@@ -435,22 +474,10 @@ class CItemPrototype extends CItemGeneral {
 
 		// Link item prototypes to application prototypes.
 		if ($item_application_prototypes) {
-			DB::insert('item_application_prototype', $item_application_prototypes);
+			DB::insertBatch('item_application_prototype', $item_application_prototypes);
 		}
 
 		$this->createItemPreprocessing($items);
-
-// TODO: REMOVE info
-		$itemHosts = $this->get([
-			'itemids' => $itemids,
-			'output' => ['name'],
-			'selectHosts' => ['name'],
-			'nopermissions' => true
-		]);
-		foreach ($itemHosts as $item) {
-			$host = reset($item['hosts']);
-			info(_s('Created: Item prototype "%1$s" on "%2$s".', $item['name'], $host['name']));
-		}
 	}
 
 	protected function updateReal($items) {
@@ -486,7 +513,7 @@ class CItemPrototype extends CItemGeneral {
 
 		if (!empty($itemidsWithApplications)) {
 			DB::delete('items_applications', ['itemid' => $itemidsWithApplications]);
-			DB::insert('items_applications', $itemApplications);
+			DB::insertBatch('items_applications', $itemApplications);
 		}
 
 		// application prototypes that are no longer linked to items will be deleted from database
@@ -626,7 +653,7 @@ class CItemPrototype extends CItemGeneral {
 
 				// Create new application prototypes, get new IDs.
 				if ($application_prototypes_to_create) {
-					$ids = DB::insert('application_prototype', $application_prototypes_to_create);
+					$ids = DB::insertBatch('application_prototype', $application_prototypes_to_create);
 
 					foreach ($ids as $id) {
 						$new_records[] = [
@@ -681,19 +708,6 @@ class CItemPrototype extends CItemGeneral {
 		}
 
 		$this->updateItemPreprocessing($items);
-
-// TODO: REMOVE info
-		$itemHosts = $this->get([
-			'itemids' => $itemids,
-			'output' => ['name'],
-			'selectHosts' => ['name'],
-			'nopermissions' => true
-		]);
-
-		foreach ($itemHosts as $item) {
-			$host = reset($item['hosts']);
-			info(_s('Updated: Item prototype "%1$s" on "%2$s".', $item['name'], $host['name']));
-		}
 	}
 
 	/**
@@ -707,23 +721,100 @@ class CItemPrototype extends CItemGeneral {
 		$items = zbx_toArray($items);
 
 		$this->checkInput($items, true);
-		$this->validateDependentItems($items, API::ItemPrototype());
+		$this->validateDependentItems($items, __METHOD__);
 
-		$dbItems = $this->get([
-			'output' => ['type', 'master_itemid'],
+		$db_items = $this->get([
+			'output' => ['type', 'master_itemid', 'authtype', 'allow_traps', 'retrieve_mode'],
 			'itemids' => zbx_objectValues($items, 'itemid'),
 			'editable' => true,
 			'preservekeys' => true
 		]);
 
-		$items = $this->extendFromObjects(zbx_toHash($items, 'itemid'), $dbItems, ['type']);
+		$items = $this->extendFromObjects(zbx_toHash($items, 'itemid'), $db_items, ['type']);
+
+		$defaults = DB::getDefaults('items');
+		$clean = [
+			ITEM_TYPE_HTTPAGENT => [
+				'url' => '',
+				'query_fields' => '',
+				'timeout' => $defaults['timeout'],
+				'status_codes' => '',
+				'follow_redirects' => $defaults['follow_redirects'],
+				'request_method' => $defaults['request_method'],
+				'allow_traps' => $defaults['allow_traps'],
+				'post_type' => $defaults['post_type'],
+				'http_proxy' => '',
+				'headers' => '',
+				'retrieve_mode' => $defaults['retrieve_mode'],
+				'output_format' => $defaults['output_format'],
+				'ssl_key_password' => '',
+				'verify_peer' => $defaults['verify_peer'],
+				'verify_host' => $defaults['verify_host'],
+				'ssl_cert_file' => '',
+				'ssl_key_file' => '',
+				'authtype' => $defaults['authtype'],
+				'username' => '',
+				'password' => '',
+				'posts' => ''
+			]
+		];
+
+		$json = new CJson();
 
 		foreach ($items as &$item) {
-			if ($item['type'] != ITEM_TYPE_DEPENDENT && $dbItems[$item['itemid']]['master_itemid']) {
+			$type_change = ($item['type'] != $db_items[$item['itemid']]['type']);
+
+			if ($item['type'] != ITEM_TYPE_DEPENDENT && $db_items[$item['itemid']]['master_itemid']) {
 				$item['master_itemid'] = null;
 			}
 			elseif (!array_key_exists('master_itemid', $item)) {
-				$item['master_itemid'] = $dbItems[$item['itemid']]['master_itemid'];
+				$item['master_itemid'] = $db_items[$item['itemid']]['master_itemid'];
+			}
+
+			if ($type_change && $db_items[$item['itemid']]['type'] == ITEM_TYPE_HTTPAGENT) {
+				$item = array_merge($item, $clean[ITEM_TYPE_HTTPAGENT]);
+
+				if ($item['type'] != ITEM_TYPE_SSH) {
+					$item['authtype'] = $defaults['authtype'];
+					$item['username'] = '';
+					$item['password'] = '';
+				}
+
+				if ($item['type'] != ITEM_TYPE_TRAPPER) {
+					$item['trapper_hosts'] = '';
+				}
+			}
+
+			if ($item['type'] == ITEM_TYPE_HTTPAGENT) {
+				// Clean username and password on authtype change to HTTPTEST_AUTH_NONE.
+				if (array_key_exists('authtype', $item) && $item['authtype'] == HTTPTEST_AUTH_NONE
+						&& $item['authtype'] != $db_items[$item['itemid']]['authtype']) {
+					$item['username'] = '';
+					$item['password'] = '';
+				}
+
+				if (array_key_exists('allow_traps', $item) && $item['allow_traps'] == HTTPCHECK_ALLOW_TRAPS_OFF
+						&& $item['allow_traps'] != $db_items[$item['itemid']]['allow_traps']) {
+					$item['trapper_hosts'] = '';
+				}
+
+				if (array_key_exists('query_fields', $item) && is_array($item['query_fields'])) {
+					$item['query_fields'] = $item['query_fields'] ? $json->encode($item['query_fields']) : '';
+				}
+
+				if (array_key_exists('headers', $item) && is_array($item['headers'])) {
+					$item['headers'] = $this->headersArrayToString($item['headers']);
+				}
+
+				if (array_key_exists('request_method', $item) && $item['request_method'] == HTTPCHECK_REQUEST_HEAD
+						&& !array_key_exists('retrieve_mode', $item)
+						&& $db_items[$item['itemid']]['retrieve_mode'] != HTTPTEST_STEP_RETRIEVE_MODE_HEADERS) {
+					$item['retrieve_mode'] = HTTPTEST_STEP_RETRIEVE_MODE_HEADERS;
+				}
+			}
+			else {
+				$item['query_fields'] = '';
+				$item['headers'] = '';
 			}
 		}
 		unset($item);
@@ -753,8 +844,7 @@ class CItemPrototype extends CItemGeneral {
 			'itemids' => $prototypeids,
 			'editable' => true,
 			'preservekeys' => true,
-			'output' => API_OUTPUT_EXTEND,
-			'selectHosts' => ['name']
+			'output' => API_OUTPUT_EXTEND
 		];
 		$delItemPrototypes = $this->get($options);
 
@@ -789,7 +879,6 @@ class CItemPrototype extends CItemGeneral {
 			$db_dependent_items = $this->get([
 				'output' => ['itemid', 'name'],
 				'filter' => ['type' => ITEM_TYPE_DEPENDENT, 'master_itemid' => array_keys($db_dependent_items)],
-				'selectHosts' => ['name'],
 				'preservekeys' => true
 			]);
 			$db_dependent_items = array_diff_key($db_dependent_items, $dependent_itemprototypes);
@@ -798,16 +887,6 @@ class CItemPrototype extends CItemGeneral {
 		$dependent_itemprototypeids = array_keys($dependent_itemprototypes);
 		$childPrototypeids += array_combine($dependent_itemprototypeids, $dependent_itemprototypeids);
 
-		$options = [
-			'output' => API_OUTPUT_EXTEND,
-			'itemids' => $childPrototypeids,
-			'nopermissions' => true,
-			'preservekeys' => true,
-			'selectHosts' => ['name']
-		];
-		$delItemPrototypesChildren = $this->get($options);
-
-		$delItemPrototypes = array_merge($delItemPrototypes, $delItemPrototypesChildren);
 		$prototypeids = array_merge($prototypeids, $childPrototypeids);
 
 		// Delete graphs or leave them if graphs still have at least one item prototype.
@@ -888,13 +967,6 @@ class CItemPrototype extends CItemGeneral {
 // ITEM PROTOTYPES
 		DB::delete('items', ['itemid' => $prototypeids]);
 
-
-// TODO: remove info from API
-		$delItemPrototypes = zbx_toHash($delItemPrototypes, 'itemid') + $dependent_itemprototypes;
-		foreach ($delItemPrototypes as $item) {
-			$host = reset($item['hosts']);
-			info(_s('Deleted: Item prototype "%1$s" on "%2$s".', $item['name'], $host['name']));
-		}
 		$prototypeids = array_map('strval', $prototypeids);
 
 		return ['prototypeids' => $prototypeids];
@@ -965,15 +1037,15 @@ class CItemPrototype extends CItemGeneral {
 		$data['templateids'] = zbx_toArray($data['templateids']);
 		$data['hostids'] = zbx_toArray($data['hostids']);
 
-		$selectFields = [];
-		foreach ($this->fieldRules as $key => $rules) {
-			if (!isset($rules['system']) && !isset($rules['host'])) {
-				$selectFields[] = $key;
+		$output = [];
+		foreach ($this->fieldRules as $field_name => $rules) {
+			if (!array_key_exists('system', $rules) && !array_key_exists('host', $rules)) {
+				$output[] = $field_name;
 			}
 		}
 
-		$items = $this->get([
-			'output' => $selectFields,
+		$tpl_items = $this->get([
+			'output' => $output,
 			'selectApplications' => ['applicationid'],
 			'selectApplicationPrototypes' => ['name'],
 			'selectPreprocessing' => ['type', 'params'],
@@ -981,11 +1053,30 @@ class CItemPrototype extends CItemGeneral {
 			'preservekeys' => true
 		]);
 
-		foreach ($items as $inum => $item) {
-			$items[$inum]['applications'] = zbx_objectValues($item['applications'], 'applicationid');
-		}
+		$json = new CJson();
 
-		$this->inherit($items, $data['hostids']);
+		foreach ($tpl_items as &$tpl_item) {
+			$tpl_item['applications'] = zbx_objectValues($tpl_item['applications'], 'applicationid');
+
+			if ($tpl_item['type'] == ITEM_TYPE_HTTPAGENT) {
+				if (array_key_exists('query_fields', $tpl_item) && is_array($tpl_item['query_fields'])) {
+					$tpl_item['query_fields'] = $tpl_item['query_fields']
+						? $json->encode($tpl_item['query_fields'])
+						: '';
+				}
+
+				if (array_key_exists('headers', $tpl_item) && is_array($tpl_item['headers'])) {
+					$tpl_item['headers'] = $this->headersArrayToString($tpl_item['headers']);
+				}
+			}
+			else {
+				$tpl_item['query_fields'] = '';
+				$tpl_item['headers'] = '';
+			}
+		}
+		unset($tpl_item);
+
+		$this->inherit($tpl_items, $data['hostids']);
 
 		return true;
 	}
@@ -1022,59 +1113,64 @@ class CItemPrototype extends CItemGeneral {
 
 	protected function inherit(array $items, array $hostids = null) {
 		if (!$items) {
-			return true;
-		}
-
-		// fetch the corresponding discovery rules for the child items
-		$ruleids = [];
-		$dbResult = DBselect(
-			'SELECT i.itemid AS ruleid,id.itemid,i.hostid'.
-			' FROM items i,item_discovery id'.
-			' WHERE i.templateid=id.parent_itemid'.
-				' AND '.dbConditionInt('id.itemid', zbx_objectValues($items, 'itemid'))
-		);
-		while ($rule = DBfetch($dbResult)) {
-			if (!isset($ruleids[$rule['itemid']])) {
-				$ruleids[$rule['itemid']] = [];
-			}
-			$ruleids[$rule['itemid']][$rule['hostid']] = $rule['ruleid'];
+			return;
 		}
 
 		// prepare the child items
-		$newItems = $this->prepareInheritedItems($items, $hostids);
-		if (!$items) {
-			return true;
+		$new_items = $this->prepareInheritedItems($items, $hostids);
+		if (!$new_items) {
+			return;
 		}
 
-		$insertItems = [];
-		$updateItems = [];
-		foreach ($newItems as $newItem) {
-			if (isset($newItem['itemid'])) {
-				unset($newItem['ruleid']);
-				$updateItems[] = $newItem;
+		$ins_items = [];
+		$upd_items = [];
+		foreach ($new_items as $new_item) {
+			if (array_key_exists('itemid', $new_item)) {
+				unset($new_item['ruleid']);
+				$upd_items[] = $new_item;
 			}
 			else {
 				// set the corresponding discovery rule id for the new items
-				$newItem['ruleid'] = $ruleids[$newItem['templateid']][$newItem['hostid']];
-				$newItem['flags'] = ZBX_FLAG_DISCOVERY_PROTOTYPE;
-				$insertItems[] = $newItem;
+				$ins_items[] = $new_item;
 			}
 		}
 
 		// save the new items
-		if ($insertItems) {
-			$this->createReal($insertItems);
+		if ($ins_items) {
+			$this->createReal($ins_items);
 		}
 
-		if ($updateItems) {
-			$this->updateReal($updateItems);
+		if ($upd_items) {
+			$this->updateReal($upd_items);
 		}
 
+		$new_items = array_merge($upd_items, $ins_items);
+
 		// Update master_itemid for inserted or updated inherited dependent items.
-		$this->inheritDependentItems(array_merge($updateItems, $insertItems));
+		$this->inheritDependentItems($new_items);
 
-		// propagate the inheritance to the children
-		$this->inherit(array_merge($insertItems, $updateItems));
+		// Inheriting items from the templates.
+		$tpl_items = DBselect(
+			'SELECT i.itemid'.
+			' FROM items i,hosts h'.
+			' WHERE i.hostid=h.hostid'.
+				' AND '.dbConditionInt('i.itemid', zbx_objectValues($new_items, 'itemid')).
+				' AND '.dbConditionInt('h.status', [HOST_STATUS_TEMPLATE])
+		);
+
+		$tpl_itemids = [];
+		while ($tpl_item = DBfetch($tpl_items)) {
+			$tpl_itemids[$tpl_item['itemid']] = true;
+		}
+
+		foreach ($new_items as $index => $new_item) {
+			if (!array_key_exists($new_item['itemid'], $tpl_itemids)) {
+				unset($new_items[$index]);
+			}
+		}
+
+		// Inheriting items from the templates.
+		$this->inherit($new_items);
 	}
 
 	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
